@@ -1,6 +1,6 @@
 #!/bin/sh
 #
-# Copyright (c) 2021  StorPool <support@storpool.com>
+# Copyright (c) 2021, 2022  StorPool <support@storpool.com>
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -45,28 +45,29 @@ detect_from_os_release()
 		return
 	fi
 
-	local output="$(unset ID VERSION_ID; . /etc/os-release; printf -- '%s::%s\n' "$ID" "$VERSION_ID")"
+	local output os_id
+	output="$(unset ID VERSION_ID; . /etc/os-release; printf -- '%s::%s\n' "$ID" "$VERSION_ID")"
 	local os_id="${output%%::*}" version_id="${output#*::}"
 	if [ -z "$os_id" ] || [ -z "$version_id" ]; then
 		return
 	fi
 	
-	if [ "$os_id" = 'alma' ] && printf -- '%s\n' "$version_id" | grep -Eqe '^8(?:$|\.[4-9]|\.[1-9][0-9])'; then
+	if [ "$os_id" = 'alma' ] && printf -- '%s\n' "$version_id" | grep -Eqe '^8($|\.[4-9]|\.[1-9][0-9])'; then
 		printf -- '%s\n' 'ALMA8'
 		return
 	fi
 	
-	if [ "$os_id" = 'centos' ] && printf -- '%s\n' "$version_id" | grep -Eqe '^6(?:$|\.[0-9])'; then
+	if [ "$os_id" = 'centos' ] && printf -- '%s\n' "$version_id" | grep -Eqe '^6($|\.[0-9])'; then
 		printf -- '%s\n' 'CENTOS6'
 		return
 	fi
 	
-	if [ "$os_id" = 'centos' ] && printf -- '%s\n' "$version_id" | grep -Eqe '^7(?:$|\.[0-9])'; then
+	if [ "$os_id" = 'centos' ] && printf -- '%s\n' "$version_id" | grep -Eqe '^7($|\.[0-9])'; then
 		printf -- '%s\n' 'CENTOS7'
 		return
 	fi
 	
-	if [ "$os_id" = 'centos' ] && printf -- '%s\n' "$version_id" | grep -Eqe '^8(?:$|\.[4-9]|\.[1-9][0-9])'; then
+	if [ "$os_id" = 'centos' ] && printf -- '%s\n' "$version_id" | grep -Eqe '^8($|\.[4-9]|\.[1-9][0-9])'; then
 		printf -- '%s\n' 'CENTOS8'
 		return
 	fi
@@ -91,17 +92,17 @@ detect_from_os_release()
 		return
 	fi
 	
-	if [ "$os_id" = 'ol' ] && printf -- '%s\n' "$version_id" | grep -Eqe '^7(?:$|\.[0-9])'; then
+	if [ "$os_id" = 'ol' ] && printf -- '%s\n' "$version_id" | grep -Eqe '^7($|\.[0-9])'; then
 		printf -- '%s\n' 'ORACLE7'
 		return
 	fi
 	
-	if [ "$os_id" = 'rhel' ] && printf -- '%s\n' "$version_id" | grep -Eqe '^8(?:$|\.[4-9]|\.[1-9][0-9])'; then
+	if [ "$os_id" = 'rhel' ] && printf -- '%s\n' "$version_id" | grep -Eqe '^8($|\.[4-9]|\.[1-9][0-9])'; then
 		printf -- '%s\n' 'RHEL8'
 		return
 	fi
 	
-	if [ "$os_id" = 'rocky' ] && printf -- '%s\n' "$version_id" | grep -Eqe '^8(?:$|\.[4-9]|\.[1-9][0-9])'; then
+	if [ "$os_id" = 'rocky' ] && printf -- '%s\n' "$version_id" | grep -Eqe '^8($|\.[4-9]|\.[1-9][0-9])'; then
 		printf -- '%s\n' 'ROCKY8'
 		return
 	fi
@@ -126,6 +127,11 @@ detect_from_os_release()
 		return
 	fi
 	
+	if [ "$os_id" = 'ubuntu' ] && printf -- '%s\n' "$version_id" | grep -Eqe '^22\.04$'; then
+		printf -- '%s\n' 'UBUNTU2204'
+		return
+	fi
+	
 }
 
 cmd_detect()
@@ -199,6 +205,11 @@ cmd_detect()
 		return
 	fi
 	
+	if [ -r '/etc/os-release' ] && grep -Eqe '^PRETTY_NAME=.*(Ubuntu[[:space:]]+22\.04|Mint[[:space:]]+22)' -- '/etc/os-release'; then
+		printf -- '%s\n' 'UBUNTU2204'
+		return
+	fi
+	
 	if [ -r '/etc/os-release' ] && grep -Eqe '^PRETTY_NAME=.*Debian[[:space:]]+GNU/Linux[[:space:]]+(stretch|9)([[:space:]]|/)' -- '/etc/os-release'; then
 		printf -- '%s\n' 'DEBIAN9'
 		return
@@ -1789,7 +1800,7 @@ show_UBUNTU2110()
     "LIBSSL": "libssl1.1",
     "MCELOG": "bash"
   },
-  "parent": "DEBIAN12",
+  "parent": "UBUNTU2204",
   "repo": {
     "codename": "impish",
     "keyring": "debian/repo/storpool-keyring.gpg",
@@ -1804,6 +1815,117 @@ show_UBUNTU2110()
 EOVARIANT_JSON
 }
 
+show_UBUNTU2204()
+{
+	cat <<'EOVARIANT_JSON'
+  {
+  "builder": {
+    "alias": "ubuntu-22.04",
+    "base_image": "ubuntu:jammy",
+    "branch": "ubuntu/jammy",
+    "kernel_package": "linux-headers",
+    "utf8_locale": "C.UTF-8"
+  },
+  "commands": {
+    "package": {
+      "install": [
+        "env",
+        "DEBIAN_FRONTEND=noninteractive",
+        "apt-get",
+        "-q",
+        "-y",
+        "--no-install-recommends",
+        "install",
+        "--"
+      ],
+      "list_all": [
+        "dpkg-query",
+        "-W",
+        "-f",
+        "${Package}\\t${Version}\\t${Architecture}\\t${db:Status-Abbrev}\\n",
+        "--"
+      ],
+      "purge": [
+        "env",
+        "DEBIAN_FRONTEND=noninteractive",
+        "apt-get",
+        "-q",
+        "-y",
+        "purge",
+        "--"
+      ],
+      "remove": [
+        "env",
+        "DEBIAN_FRONTEND=noninteractive",
+        "apt-get",
+        "-q",
+        "-y",
+        "remove",
+        "--"
+      ],
+      "remove_impl": [
+        "env",
+        "DEBIAN_FRONTEND=noninteractive",
+        "dpkg",
+        "-r",
+        "--"
+      ],
+      "update_db": [
+        "apt-get",
+        "-q",
+        "-y",
+        "update"
+      ]
+    },
+    "pkgfile": {
+      "dep_query": [
+        "sh",
+        "-c",
+        "dpkg-deb -f -- \"$pkg\" 'Depends' | sed -e 's/ *, */,/g' | tr ',' \"\\n\""
+      ],
+      "install": [
+        "sh",
+        "-c",
+        "env DEBIAN_FRONTEND=noninteractive apt-get install --no-install-recommends --reinstall -y -o DPkg::Options::=--force-confnew -- $packages"
+      ]
+    }
+  },
+  "descr": "Ubuntu 22.04 LTS (Jammy Jellyfish)",
+  "detect": {
+    "filename": "/etc/os-release",
+    "os_id": "ubuntu",
+    "os_version_regex": "^22\\.04$",
+    "regex": "^ PRETTY_NAME= .* (?: Ubuntu \\s+ 22 \\. 04 | Mint \\s+ 22 ) "
+  },
+  "family": "debian",
+  "file_ext": "deb",
+  "initramfs_flavor": "update-initramfs",
+  "min_sys_python": "3.9",
+  "name": "UBUNTU2204",
+  "package": {
+    "BINDINGS_PYTHON": "python3",
+    "BINDINGS_PYTHON_CONFGET": "python3-confget",
+    "BINDINGS_PYTHON_SIMPLEJSON": "python3-simplejson",
+    "CGROUP": "cgroup-tools",
+    "CPUPOWER": "linux-tools-generic",
+    "LIBSSL": "libssl1.1",
+    "MCELOG": "bash"
+  },
+  "parent": "DEBIAN12",
+  "repo": {
+    "codename": "jammy",
+    "keyring": "debian/repo/storpool-keyring.gpg",
+    "req_packages": [
+      "ca-certificates"
+    ],
+    "sources": "debian/repo/storpool.sources",
+    "vendor": "ubuntu"
+  },
+  "systemd_lib": "lib/systemd/system"
+}
+EOVARIANT_JSON
+}
+
 
 cmd_show_all()
 {
@@ -1832,6 +1954,7 @@ cmd_show_all()
     "UBUNTU1804",
     "UBUNTU2004",
     "UBUNTU2110",
+    "UBUNTU2204",
     "DEBIAN9",
     "DEBIAN10",
     "DEBIAN11",
@@ -1885,11 +2008,14 @@ EOPROLOGUE
   echo ','
   printf -- '    "%s": ' 'UBUNTU2110'
   show_UBUNTU2110
+  echo ','
+  printf -- '    "%s": ' 'UBUNTU2204'
+  show_UBUNTU2204
   
 
 	cat <<'EOEPILOGUE'
   },
-  "version": "2.1.1"
+  "version": "2.3.2"
 }
 EOEPILOGUE
 }
@@ -1913,7 +2039,7 @@ EOPROLOGUE
 
 	cat <<'EOEPILOGUE'
   ,
-  "version": "2.1.1"
+  "version": "2.3.2"
 }
 EOEPILOGUE
 }
@@ -1925,7 +2051,8 @@ cmd_show_current()
 		exit 1
 	fi
 
-	local variant="$(cmd_detect)"
+	local variant
+	variant="$(cmd_detect)"
 	[ -n "$variant" ]
 
 	show_variant "$variant"
@@ -3038,6 +3165,71 @@ fi
 			esac
 			;;
 		
+		UBUNTU2204)
+			case "$cmd_cat" in
+				
+				package)
+					case "$cmd_item" in
+						
+						install)
+							$noop 'env' 'DEBIAN_FRONTEND=noninteractive' 'apt-get' '-q' '-y' '--no-install-recommends' 'install' '--'  "$@"
+							;;
+						
+						list_all)
+							$noop 'dpkg-query' '-W' '-f' '${Package}\t${Version}\t${Architecture}\t${db:Status-Abbrev}\n' '--'  "$@"
+							;;
+						
+						purge)
+							$noop 'env' 'DEBIAN_FRONTEND=noninteractive' 'apt-get' '-q' '-y' 'purge' '--'  "$@"
+							;;
+						
+						remove)
+							$noop 'env' 'DEBIAN_FRONTEND=noninteractive' 'apt-get' '-q' '-y' 'remove' '--'  "$@"
+							;;
+						
+						remove_impl)
+							$noop 'env' 'DEBIAN_FRONTEND=noninteractive' 'dpkg' '-r' '--'  "$@"
+							;;
+						
+						update_db)
+							$noop 'apt-get' '-q' '-y' 'update'  "$@"
+							;;
+						
+
+						*)
+							echo "Invalid command '$cmd_item' in the '$cmd_cat' category" 1>&2
+							exit 1
+							;;
+					esac
+					;;
+				
+				pkgfile)
+					case "$cmd_item" in
+						
+						dep_query)
+							$noop 'sh' '-c' 'dpkg-deb -f -- "$pkg" 'Depends' | sed -e 's/ *, */,/g' | tr ',' "\n"'  "$@"
+							;;
+						
+						install)
+							$noop 'sh' '-c' 'env DEBIAN_FRONTEND=noninteractive apt-get install --no-install-recommends --reinstall -y -o DPkg::Options::=--force-confnew -- $packages'  "$@"
+							;;
+						
+
+						*)
+							echo "Invalid command '$cmd_item' in the '$cmd_cat' category" 1>&2
+							exit 1
+							;;
+					esac
+					;;
+				
+
+				*)
+					echo "Invalid command category '$cmd_cat'" 1>&2
+					exit 1
+					;;
+			esac
+			;;
+		
 
 		*)
 			echo "Internal error: invalid variant '$name'" 1>&2
@@ -3103,12 +3295,14 @@ repo_add_yum()
 
 	yum --disablerepo='storpool-*' install -q -y ca-certificates
 
-	local yumbase="$(basename -- "$yumdef")"
-	local repofile="$(repo_add_extension "$yumbase" "$repotype")"
+	local yumbase repofile
+	yumbase="$(basename -- "$yumdef")"
+	repofile="$(repo_add_extension "$yumbase" "$repotype")"
 	[ -n "$repofile" ]
 	copy_file "$vdir/$repofile" /etc/yum.repos.d
 
-	local keybase="$(basename -- "$keyring")"
+	local keybase
+	keybase="$(basename -- "$keyring")"
 	copy_file "$vdir/$keybase" /etc/pki/rpm-gpg
 
 	if [ -n "$(command -v rpmkeys)" ]; then
@@ -3125,12 +3319,14 @@ repo_add_deb()
 	# $packages is unquoted on purpose: there may be more than one.
 	run_command "$name" '' package.install $packages
 
-	local srcbase="$(basename -- "$srcdef")"
-	local repofile="$(repo_add_extension "$srcbase" "$repotype")"
+	local srcbase repofile
+	srcbase="$(basename -- "$srcdef")"
+	repofile="$(repo_add_extension "$srcbase" "$repotype")"
 	[ -n "$repofile" ]
 	copy_file "$vdir/$repofile" /etc/apt/sources.list.d
 
-	local keybase="$(basename -- "$keyring")"
+	local keybase
+	keybase="$(basename -- "$keyring")"
 	copy_file "$vdir/$keybase" /usr/share/keyrings
 
 	apt-get update
@@ -3138,7 +3334,8 @@ repo_add_deb()
 
 cmd_repo_add()
 {
-	local repodir="$(dirname -- "$0")" repotype='contrib'
+	local repodir repotype='contrib'
+	repodir="$(dirname -- "$0")"
 	local o
 	while getopts 'd:t:' o; do
 		case "$o" in
@@ -3162,7 +3359,8 @@ cmd_repo_add()
 		exit 1
 	fi
 
-	local variant="$(cmd_detect)"
+	local variant
+	variant="$(cmd_detect)"
 	local vdir="$repodir/$variant"
 	if [ ! -d "$vdir" ]; then
 		echo "No $vdir directory" 1>&2
@@ -3261,6 +3459,12 @@ cmd_repo_add()
 			
 			;;
 		
+		UBUNTU2204)
+			
+			repo_add_deb 'UBUNTU2204' "$vdir" "$repotype" 'debian/repo/storpool.sources' 'debian/repo/storpool-keyring.gpg' 'ca-certificates'
+			
+			;;
+		
 	esac
 
 	install -o root -g root -m 755 -- "$0" /usr/sbin/sp_variant
@@ -3283,7 +3487,8 @@ cmd_command_run()
 	done
 	shift "$((OPTIND - 1))"
 
-	local variant="$(cmd_detect)"
+	local variant
+	variant="$(cmd_detect)"
 	[ -n "$variant" ]
 
 	run_command "$variant" "$noop" "$@"
@@ -3306,7 +3511,7 @@ cmd_command()
 
 cmd_features()
 {
-	echo 'Features: format=1.3 version=2.1.1'
+	echo 'Features: format=1.3 version=2.3.2'
 }
 
 case "$1" in
@@ -3414,6 +3619,10 @@ case "$1" in
 				show_variant 'UBUNTU2110'
 				;;
 			
+			UBUNTU2204)
+				show_variant 'UBUNTU2204'
+				;;
+			
 
 			*)
 				usage 1>&2
